@@ -1,4 +1,4 @@
-// Copyright 2013 Daniel Parker
+// Copyright 2013-2023 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -14,7 +14,7 @@ namespace jsoncons {
 struct sorted_policy;
                         
 template <class CharT, 
-          class ImplementationPolicy = sorted_policy, 
+          class Policy = sorted_policy, 
           class Allocator = std::allocator<CharT>>
 class basic_json;
 
