@@ -1,4 +1,4 @@
-// Copyright 2017 Daniel Parker
+// Copyright 2013-2023 Daniel Parker
 // Distributed under the Boost license, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -19,7 +19,7 @@
 #include <jsoncons_ext/msgpack/msgpack_type.hpp>
 #include <jsoncons_ext/msgpack/msgpack_error.hpp>
 #include <jsoncons_ext/msgpack/msgpack_options.hpp>
-#include <jsoncons/json_visitor2.hpp>
+#include <jsoncons/item_event_visitor.hpp>
 
 namespace jsoncons { namespace msgpack {
 
@@ -66,7 +66,7 @@ public:
     template <class Sourceable>
     basic_msgpack_parser(Sourceable&& source,
                          const msgpack_decode_options& options = msgpack_decode_options(),
-                         const Allocator alloc = Allocator())
+                         const Allocator& alloc = Allocator())
        : source_(std::forward<Sourceable>(source)),
          options_(options),
          more_(true), 
@@ -86,10 +86,20 @@ public:
 
     void reset()
     {
-        state_stack_.clear();
-        state_stack_.emplace_back(parse_mode::root,0);
         more_ = true;
         done_ = false;
+        text_buffer_.clear();
+        bytes_buffer_.clear();
+        state_stack_.clear();
+        state_stack_.emplace_back(parse_mode::root,0);
+        nesting_depth_ = 0;
+    }
+
+    template <class Sourceable>
+    void reset(Sourceable&& source)
+    {
+        source_ = std::forward<Sourceable>(source);
+        reset();
     }
 
     bool done() const
@@ -112,7 +122,7 @@ public:
         return source_.position();
     }
 
-    void parse(json_visitor2& visitor, std::error_code& ec)
+    void parse(item_event_visitor& visitor, std::error_code& ec)
     {
         while (!done_ && more_)
         {
@@ -187,7 +197,7 @@ public:
     }
 private:
 
-    void read_item(json_visitor2& visitor, std::error_code& ec)
+    void read_item(item_event_visitor& visitor, std::error_code& ec)
     {
         if (source_.is_error())
         {
@@ -604,7 +614,7 @@ private:
         }
     }
 
-    void begin_array(json_visitor2& visitor, uint8_t type, std::error_code& ec)
+    void begin_array(item_event_visitor& visitor, uint8_t type, std::error_code& ec)
     {
         if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
         {
@@ -621,7 +631,7 @@ private:
         more_ = visitor.begin_array(length, semantic_tag::none, *this, ec);
     }
 
-    void end_array(json_visitor2& visitor, std::error_code& ec)
+    void end_array(item_event_visitor& visitor, std::error_code& ec)
     {
         --nesting_depth_;
 
@@ -629,7 +639,7 @@ private:
         state_stack_.pop_back();
     }
 
-    void begin_object(json_visitor2& visitor, uint8_t type, std::error_code& ec)
+    void begin_object(item_event_visitor& visitor, uint8_t type, std::error_code& ec)
     {
         if (JSONCONS_UNLIKELY(++nesting_depth_ > options_.max_nesting_depth()))
         {
@@ -646,7 +656,7 @@ private:
         more_ = visitor.begin_object(length, semantic_tag::none, *this, ec);
     }
 
-    void end_object(json_visitor2& visitor, std::error_code& ec)
+    void end_object(item_event_visitor& visitor, std::error_code& ec)
     {
         --nesting_depth_;
         more_ = visitor.end_object(*this, ec);
